-- Batch Renderer 1.0
-- by Jarek Dukat
-- http://www.nautilus.com.pl/jarek
-- jarek@nautilus.com.pl

global joben, jobsc, jobcam, jobcamn, jobrng, jobtyp, jobfile, jobcount

rollout brabout " About " (
	label ab1 "Batch Renderer 1.0"
	label ab2 "by Jarek Dukat"
	label ab3 "www.nautilus.com.pl/jarek"
	label ab4 " jarek@nautilus.com.pl"
)

utility br "Batch Renderer 1.0" (
	spinner brcur "Current Job: " range:[1,1,1] type:#integer fieldwidth:35 align:#left across:2
	label brall "/ 1" align:#right
	checkbox bren "Render This Job"
	button brscene "Pick Scene" width:104 across:2 offset:[-3,0]
	button brload "LOAD" width:40 offset:[18,0]
	pickbutton brcam "Pick Camera" filter:validcam width:145 enabled:false
	spinner brstart "Start Frame:" range:[0,100,0] type:#integer enabled:false
	spinner brend "End Frame:" range:[0,100,100] type:#integer enabled:false
	radiobuttons brtype "Render Using:" labels:#("Production", "Draft") enabled:false
	label brlab "Save File:" across:2 align:#left offset:[0,3]
	button brpick "Browse" align:#right height:16
	edittext brfile fieldwidth:156 offset:[-15,0]

	group "Queue Manager" (
		button bradd "Add New Job" width:120 offset:[0,5]
		button brcpy "Copy Current Job" width:120 offset:[0,-5]
		button brres "Reset Current Job" width:120 offset:[0,-5]
		button brdel "Delete Current Job" width:120 enabled:false offset:[0,-5]
		button brtr "Test Render (1 frame)" width:120 offset:[0,-5] tooltip:"Render first frame of current job"
		button brresall "Reset All Jobs" width:120
		button brrend "RENDER QUEUE!" height:25 width:140 offset:[0,3]
		label beempty "" offset:[0,-10]
	)

	on bradd pressed do addjob()
	on brcpy pressed do copyjob()
	on brres pressed do resetjob()
	on brdel pressed do deljob()
	on brtr pressed do testjob()
	on brresall pressed do if querybox "Are you sure?" title:"Reset Queue" do (
		for i = 1 to jobcount - 1 do deljob()
		resetjob()
	)
	on brcur changed job do dispjob()

	on bren changed ch do joben[brcur.value]=ch
	on brscene pressed do openscene()
	on brload pressed do loadscene()
	on brcam picked cam do setcam cam
	on brpick pressed do filesave()
	on brstart changed ch do jobrng[brcur.value][3] = ch
	on brend changed ch do jobrng[brcur.value][4] = ch
	on brtype changed ch do jobtyp[brcur.value] = if ch == 1 then #production else #draft
	on brfile entered f do jobfile[brcur.value] = f

	on brrend pressed do batchrender()

	on br open do (
		addrollout brabout
		joben=#(); jobsc=#(); jobcam=#(); jobcamn=#()
		jobrng=#(); jobtyp=#(); jobfile=#()
		jobcount = 0
		addjob()
	)
	on br close do removerollout brabout
)

fn validcam obj = superclassof obj == camera

fn addjob = (
	jobcount += 1
	br.brall.text = "/ " + (jobcount as string)
	br.brcur.range = [1,jobcount, jobcount]
	br.brdel.enabled = true
	resetjob()
)

fn copyjob = (
	cur = br.brcur.value
	addjob()
	joben[jobcount] = joben[cur]; jobsc[jobcount] = jobsc[cur]; jobfile[jobcount] = jobfile[cur]
	jobcam[jobcount] = jobcam[cur]; jobcamn[jobcount] = jobcamn[cur]
	jobtyp[jobcount] = jobtyp[cur]; jobrng[jobcount] = jobrng[cur]
	dispjob()
)

fn resetjob = (
	joben[br.brcur.value] = false; jobsc[br.brcur.value] = "Pick Scene"; jobfile[br.brcur.value] = ""
	jobcam[br.brcur.value] = 0; jobcamn[br.brcur.value] = "Pick Camera"
	jobtyp[br.brcur.value] = #production; jobrng[br.brcur.value] = #(0,0,0,0)
	dispjob()
)

fn deljob = (
	c = br.brcur.value
	deleteitem joben c
	deleteitem jobsc c
	deleteitem jobfile c
	deleteitem jobcam c
	deleteitem jobcamn c
	deleteitem jobrng c
	deleteitem jobtyp c
	jobcount -= 1
	br.brall.text = "/ " + (jobcount as string)
	if c>jobcount do c -= 1
	br.brcur.range = [1,jobcount, c]
	if jobcount == 1 then br.brdel.enabled = false else br.brdel.enabled = true
	dispjob()
)

fn dispjob = (
	job = br.brcur.value
	br.bren.checked = joben[job]
	br.brscene.text = filenamefrompath jobsc[job]
	br.brcam.text = jobcamn[job]
	br.brtype.state = if jobtyp[job] == #production then 1 else 2
	br.brfile.text = jobfile[job]
	br.brstart.range = [jobrng[job][1], jobrng[job][2], jobrng[job][3]]
	br.brend.range = [jobrng[job][1], jobrng[job][2], jobrng[job][4]]
	
	br.brcam.enabled = if maxfilename == filenamefrompath jobsc[job] then true else false
	if br.brscene.text == "Pick Scene" then (
		br.brstart.enabled = false
		br.brend.enabled = false
		br.brtype.enabled = false
	) else (
		br.brstart.enabled = true
		br.brend.enabled = true
		br.brtype.enabled = true
	)
)

fn openscene = (
	f = getopenfilename caption:"Open MAX Scene for Batch Renderer" types:"MAX Scene (*.max)|*.max"
	if f != undefined do (
		jobsc[br.brcur.value] = f
		br.brscene.text = filenamefrompath f
		jobcam[br.brcur.value] = 0
		br.brcam.enabled = true
		br.brcam.text = "Pick Camera"
		if maxfilename != filenamefrompath f do loadscene()
		jobrng[br.brcur.value] = #(animationRange.start,animationRange.end,animationRange.start,animationRange.end)
		dispjob()
	)
)

fn loadscene = (
	loadmaxfile jobsc[br.brcur.value]
	br.brcam.enabled = true
)

fn setcam cam = (
	for i = 1 to $cameras.count do if $cameras[i] == cam do jobcam[br.brcur.value] = i
	br.brcam.text = cam.name
	jobcamn[br.brcur.value] = cam.name
)

fn filesave = (
	tit = "Select folder and filename to save by job " + (br.brcur.value as string)
	fname = getsavefilename caption:tit types:"JPEG Image (*.jpg)|*.jpg|TIFF Image (*.tif)|*.tif|TARGA Image (*.tga)|*.tga|PNG Image (*.png)|*.png|BMP Image (*.bmp)|*.bmp|RLA Image (*.rla)|*.rla|AVI VideoClip (*.avi)|*.avi"
	if fname != undefined do (
		br.brfile.text = fname
		jobfile[br.brcur.value] = fname
	)
)

fn testjob = (
	job = br.brcur.value
	if jobcam[job]!=0 then (
		if maxfilename != filenamefrompath(jobsc[job]) do loadmaxfile jobsc[job]
		r = render camera:$cameras[jobcam[job]] frame:jobrng[job][3] outputfile:jobfile[job] renderer:jobtyp[job]
	) else messagebox "Job not configured!" title:"Test Render Error"
)

fn batchrender = (
	try clearlistener() catch ()
	format "-------------------------------\nExecuting Batch Renderer Queue:\n-------------------------------\n"
	for job = 1 to jobcount do (
		if joben[job] then (
			format "Executing job % ... " job
			if jobcam[job]!=0 then (
				if maxfilename != filenamefrompath(jobsc[job]) do loadmaxfile jobsc[job]
				r = render camera:$cameras[jobcam[job]] fromframe:jobrng[job][3] toframe:jobrng[job][4] outputfile:jobfile[job] renderer:jobtyp[job]
				undisplay r
				format "done;\n"
			) else format "not configured, skipped;\n"
		) else format "Job % not enabled, skipped;\n" job
	)
	format "Batch queue finished!\n-------------------------------\n"
)
